#!/bin/sh

# Checkout specified openwrt and package versions to new directory (./openwrt) in current directory
# Add files and configuration for Mender compatability

# Date as yyyy-mm-dd or "yyyy-mm-dd hh:mm"
# If version changed, will have file version issues
 
OPENWRT_VERSION=2019-02-15

# Checkout method from: https://forum.archive.openwrt.org/viewtopic.php?id=51412

get_openwrt()
{
	# Checkout openwrt trunk and packages
	git clone https://git.openwrt.org/openwrt/openwrt.git
	cd openwrt
	./scripts/feeds update -a
	cd -
}

set_openwrt_version()
{
	# versionT  -  Update source code to revision T and then build
	#              T argument: date as yyyy-mm-dd or "yyyy-mm-dd hh:mm"
	#              first "git pull --depth=500" to deepen git history (shallow)
	#              use "git checkout master" for each feed to get back to HEAD

	echo "...update main source..."
	[ -d .svn ] && svn up -r {"$1"}
	[ -d .git ] && git checkout `git rev-list -n 1 --before="$1" master`
	echo "...update feeds..."
	(cd feeds/luci     ; git checkout `git rev-list -n 1 --before="$1" master`)
	(cd feeds/packages ; git checkout `git rev-list -n 1 --before="$1" master`)
	(cd feeds/routing  ; git checkout `git rev-list -n 1 --before="$1" master`)
	#(cd feeds/telephony  ; git checkout `git rev-list -n 1 --before="$1" master`)
	echo "...install feeds..."
	./scripts/feeds install -a
	#echo "...make defconfig..."
	#make defconfig
}

rm -rf openwrt
get_openwrt
cd openwrt; set_openwrt_version $OPENWRT_VERSION
