#!/bin/sh
# Mender disk image is created from openwrt disk image
# Run from OpenWrt $TOPDIR as root

# TODO: dd/flash is SLOW, increase dd bs (write block size)

# Need to alter these variables for your distribution / infrastructure
BASE_NAME=mender-neo2-openwrt
DIST_DEST_DIR=/mnt/cifs/OpenWrt/NEO2/trunk
MENDER_SERVER=yours
MENDER_SERVER_UID=yours
MENDER_SERVER_PASS=yours
# Size of Disk (8GB)
# If you change openwrt image partion size, must also alter ORIG_P2_START_SECTOR
TOTAL_SECTORS=15523840
# End need to alter

TOPDIR=$(pwd)
WORKDIR=/tmp/mender

DATE=$(date +%Y%m%d)
#DATE=20190404
OUT_IMAGE=$BASE_NAME-ext4-sdcard.img
ROOTFS_IMAGE=$BASE_NAME-ext4-rootfs.img
DIST_FILE=$BASE_NAME-$DATE
ROOTFS_FILE=$BASE_NAME-rootfs-$DATE

IN_IMAGE=openwrt-sunxi-cortexa53-sun50i-h5-nanopi-neo2-ext4-sdcard.img
SRCDIR=bin/targets/sunxi/cortexa53-glibc

DEVICE=mmcblk0
EXT4_SECTOR_SIZE=512
FAT32_BLOCK_SIZE=1024

# Partition table, u-boot reserved, partition 1 follows
RESERVED_SECTORS=2048

# Openwrt leaves space between p1(boot) and p2(rootfs), so must fdisk to determine $ORIG_P2_START_SECTOR
ORIG_P2_START_SECTOR=45056
ORIG_P2_OFFSET=$(($ORIG_P2_START_SECTOR * $EXT4_SECTOR_SIZE))

# Final Mender disk layout:
# Sector	0						Partition Table (no partition)
# Sectors	1[$RESERVED_SECTORS]				Uboot binary (no partition)
# Sectors	$P1_START_SECTOR[$BOOT_PARTITION_SECTORS]	FAT32 Boot Sector (P1)
# Sectors	$P2_START_SECTOR[$ROOTFS_PARTITION_SECTORS]	Ext4 RootFs1 (P2)
# Sectors	$P3_START_SECTOR[$ROOTFS_PARTITION_SECTORS]	Ext4 RootFs2 (P3)
# Sectors	$P4_START_SECTOR[$DATA_PARTITION_SECTORS]	Ext4 Persistant Data (P4)

get_config_value()
{
	fgrep $1 .config | cut -d "=" -f2

}

echo "To create image and flash device: $0 device name, eg: $0 sdd"
echo "To create image only: $0"

# Get disk layout from openwrt .config
BOOT_PARTITION_SIZE_MB=$(get_config_value BOOT_PARTSIZE)
BOOT_PARTITION_SECTORS=$(($BOOT_PARTITION_SIZE_MB*1024*1024/$EXT4_SECTOR_SIZE))
ROOTFS_PARTITION_SIZE_MB=$(get_config_value CONFIG_TARGET_ROOTFS_PARTSIZE)
ROOTFS_PARTITION_SECTORS=$(($ROOTFS_PARTITION_SIZE_MB*1024*1024/$EXT4_SECTOR_SIZE))
DATA_PARTITION_SECTORS=$(($TOTAL_SECTORS-$RESERVED_SECTORS-$BOOT_PARTITION_SECTORS-$ROOTFS_PARTITION_SECTORS*2))
DATA_PARTITION_SIZE_MB=$(($DATA_PARTITION_SECTORS*$EXT4_SECTOR_SIZE/1024/1024))
P2_START_SECTOR=$(($BOOT_PARTITION_SIZE_MB*2048+$RESERVED_SECTORS))
P3_START_SECTOR=$(($ROOTFS_PARTITION_SIZE_MB*2048+$P2_START_SECTOR))
P4_START_SECTOR=$(($ROOTFS_PARTITION_SIZE_MB*2048+$P3_START_SECTOR))

echo BOOT_PARTITION_SIZE_MB: $BOOT_PARTITION_SIZE_MB
echo BOOT_PARTITION_SECTORS: $BOOT_PARTITION_SECTORS
echo ROOTFS_PARTITION_SIZE_MB: $ROOTFS_PARTITION_SIZE_MB
echo ROOTFS_PARTITION_SECTORS: $ROOTFS_PARTITION_SECTORS
echo DATA_PARTITION_SIZE_MB: $DATA_PARTITION_SIZE_MB
echo DATA_PARTITION_SECTORS: $DATA_PARTITION_SECTORS
echo P2_START_SECTOR: $P2_START_SECTOR
echo P3_START_SECTOR: $P3_START_SECTOR
echo P4_START_SECTOR: $P4_START_SECTOR

# Temp
#if [ "zero" == "one" ]; then

if [ ! -d /mnt/$DEVICE ]; then
        # Create mount point
        mkdir -p /mnt/$DEVICE
fi
mkdir -p $WORKDIR
rm -rf $WORKDIR/*
umount /mnt/$DEVICE

cp -f $SRCDIR/$IN_IMAGE.gz $WORKDIR
gunzip -f $WORKDIR/$IN_IMAGE.gz
rm -f $WORKDIR/$IN_IMAGE.gz

# Get boot files from openwrt image
mount -o loop,offset=$(($RESERVED_SECTORS*$EXT4_SECTOR_SIZE)) $WORKDIR/$IN_IMAGE /mnt/$DEVICE
cp /mnt/$DEVICE/{boot.scr,dtb,uImage} $WORKDIR
sync
umount /mnt/$DEVICE

# Get u-boot binary (first 1MB = 2048 sectors of SD minus partition table - sector 0) 
dd if=$WORKDIR/$IN_IMAGE of=$WORKDIR/$OUT_IMAGE.u-boot bs=$EXT4_SECTOR_SIZE skip=1 count=$(($RESERVED_SECTORS-1))

# Get rootfs image from image
dd if=$WORKDIR/$IN_IMAGE of=$WORKDIR/$ROOTFS_IMAGE bs=$EXT4_SECTOR_SIZE skip=$ORIG_P2_START_SECTOR count=$ROOTFS_PARTITION_SECTORS

# Copy updated files to rootfs image
mount -o loop $WORKDIR/$ROOTFS_IMAGE /mnt/$DEVICE
cp Custom/boot_replace/network_new /mnt/$DEVICE/etc/config
cp Custom/boot_replace/distfeeds_new.conf /mnt/$DEVICE/etc/opkg/
echo "$BASE_NAME" > /mnt/$DEVICE/etc/release
echo "127.0.0.1 $BASE_NAME" >> /mnt/$DEVICE/etc/hosts
echo -e "/lib\n/usr/lib\n" > /mnt/$DEVICE/etc/ld.so.conf
# Shut up kernel serial console messages after boot
echo "kernel.printk = 0 4 1 7" >> /mnt/$DEVICE/etc/sysctl.conf
# Replace default opkg repository
mv -f /mnt/$DEVICE/etc/opkg/distfeeds_new.conf /mnt/$DEVICE/etc/opkg/distfeeds.conf

# Mender stuff
cp Custom/boot_replace/mender_rc.local /mnt/$DEVICE/etc/rc.local
cp Custom/boot_replace/mender_etc_config_fstab /mnt/$DEVICE/etc/config/fstab
cp Custom/boot_replace/mender_etc_fstab /mnt/$DEVICE/etc/fstab
# Add artifact-name (uniqely identifies product and version)
mkdir -p /mnt/$DEVICE/etc/mender/
echo "artifact_name=$BASE_NAME-$DATE" > /mnt/$DEVICE/etc/mender/artifact_info
# Boot files for linux partition
mkdir -p /mnt/$DEVICE/boot
cp $WORKDIR/{boot.scr,dtb,uImage} /mnt/$DEVICE/boot/
sync
umount /mnt/$DEVICE
e2fsck -y $WORKDIR/$ROOTFS_IMAGE

# Create (empty) partition table image to which all other images will be appended
dd if=/dev/zero of=$WORKDIR/$OUT_IMAGE bs=$EXT4_SECTOR_SIZE count=1

# Create boot partition image and add files - express in sectors rather than $FAT32_BLOCK_SIZE
# If want boot sector to be ext4, need to create empty image,
#   configure and recompile boot.scr, since it is checksummed
# dd if=/dev/zero of=$WORKDIR/$OUT_IMAGE.boot bs=$EXT4_SECTOR_SIZE count=$BOOT_PARTITION_SECTORS
# mkfs.ext4 -F $WORKDIR/$OUT_IMAGE.boot
mkfs.fat $WORKDIR/$OUT_IMAGE.boot -C $(($BOOT_PARTITION_SECTORS*$EXT4_SECTOR_SIZE/$FAT32_BLOCK_SIZE))
mount -o loop $WORKDIR/$OUT_IMAGE.boot /mnt/$DEVICE
cp $WORKDIR/{boot.scr,dtb,uImage} /mnt/$DEVICE/
sync
umount /mnt/$DEVICE

# Create empty ext4 auxiliary rootfs partition image
dd if=/dev/zero of=$WORKDIR/$OUT_IMAGE.aux_rootfs bs=$EXT4_SECTOR_SIZE count=$ROOTFS_PARTITION_SECTORS
mkfs.ext4 -F $WORKDIR/$OUT_IMAGE.aux_rootfs

# Create empty ext4 data partition image
dd if=/dev/zero of=$WORKDIR/$OUT_IMAGE.data bs=$EXT4_SECTOR_SIZE count=$DATA_PARTITION_SECTORS
mkfs.ext4 -F $WORKDIR/$OUT_IMAGE.data
mount -o loop $WORKDIR/$OUT_IMAGE.data /mnt/$DEVICE
# Create persistant data directories, files (mounted on /data)
mkdir -p /mnt/$DEVICE/mender
echo "device_type=$BASE_NAME" > /mnt/$DEVICE/mender/device_type
sync
umount /mnt/$DEVICE

# Get rid of dirty bit
fsck -y $WORKDIR/$OUT_IMAGE.boot

# u-boot raw flash, after partition table (sector 0)
dd of=$WORKDIR/$OUT_IMAGE bs=$EXT4_SECTOR_SIZE seek=1 if=$WORKDIR/$OUT_IMAGE.u-boot conv=notrunc
# boot partition
dd of=$WORKDIR/$OUT_IMAGE bs=$EXT4_SECTOR_SIZE seek=$RESERVED_SECTORS if=$WORKDIR/$OUT_IMAGE.boot conv=notrunc
# first rootfs partition
dd of=$WORKDIR/$OUT_IMAGE bs=$EXT4_SECTOR_SIZE seek=$P2_START_SECTOR if=$WORKDIR/$ROOTFS_IMAGE conv=notrunc
# second rootfs partition
dd of=$WORKDIR/$OUT_IMAGE bs=$EXT4_SECTOR_SIZE seek=$P3_START_SECTOR if=$WORKDIR/$OUT_IMAGE.aux_rootfs conv=notrunc
# persistant data partition
dd of=$WORKDIR/$OUT_IMAGE bs=$EXT4_SECTOR_SIZE seek=$P4_START_SECTOR if=$WORKDIR/$OUT_IMAGE.data conv=notrunc

# Create partition table
fdisk $WORKDIR/$OUT_IMAGE <<EOF
n
p
1
$RESERVED_SECTORS
+$((BOOT_PARTITION_SIZE_MB))M
n
p
2
$P2_START_SECTOR
+$((ROOTFS_PARTITION_SIZE_MB))M
n
p
3
$P3_START_SECTOR
+$((ROOTFS_PARTITION_SIZE_MB))M
n
p
4
$P4_START_SECTOR
+$((DATA_PARTITION_SECTORS-1))
t
1
c
p
w
EOF

if [ "$1" != "" ]; then
        # Flash SD card also
        umount /dev/$1*
	echo Flashing $BASE_NAME to /dev/$1
        dd if=$WORKDIR/$OUT_IMAGE of=/dev/$1 bs=100M
fi

# Create distribution images
cd $WORKDIR
rm -f $DIST_FILE.zip
rm -f $ROOTFS_FILE.zip
rm -f $DIST_DEST_DIR/$DIST_FILE.zip
zip $DIST_FILE.zip $OUT_IMAGE
zip $ROOTFS_FILE.zip $ROOTFS_IMAGE

# Copy images to server
cp $DIST_FILE.zip $DIST_DEST_DIR
#cp $ROOTFS_FILE.zip $DIST_DEST_DIR

# Temp
#fi
#cd $WORKDIR

# Create Mender rootfs artifact
mender-artifact write rootfs-image -n $BASE_NAME-$DATE -t $BASE_NAME -u $ROOTFS_IMAGE -o $BASE_NAME-ext4-rootfs-$DATE.mender
# Login and upload artifact to Mender server
mender-cli login --server $MENDER_SERVER --username $MENDER_SERVER_UID <<EOF
$MENDER_SERVER_PASS
EOF
mender-cli artifacts upload --server $MENDER_SERVER -v $BASE_NAME-ext4-rootfs-$DATE.mender
cd -

rm -rf $WORKDIR
